from __future__ import annotations
import logging

import pendulum
from minio import Minio
from minio.deleteobjects import DeleteObject
from minio.error import S3Error

from capsule import settings

logger = logging.getLogger(__name__)


# --------------------------------------------------------------------------------
class MinioServer:
    """
    Base class to handle file transfers to and from remote server.
    """

    def __init__(self, filename: str, prefix: str = '', bucket: str = settings.SYSTEM_NAME):

        if not bucket:
            raise SystemExit('No bucket name was defined. Confirm that system setup is complete.')

        self.filename = filename
        self.object_name = f"{prefix}/{filename.split('/')[-1]}"
        self.bucket = bucket.replace(' ', '').lower()
        self.prefix = prefix
        self._client = None

    def client(self) -> Minio:
        if not self._client:
            self._client = Minio(**settings.clientcfg)
        return self._client

    def upload(self) -> str:
        """
        Upload database to remote backup server.
        """
        client = self.client()

        try:
            if not client.bucket_exists(self.bucket):
                client.make_bucket(self.bucket)

            client.fput_object(
                bucket_name=self.bucket,
                object_name=self.object_name,
                file_path=self.filename,
            )

        except S3Error as e:
            logger.exception(f'Unable to upload {self.filename}: {e}')

        return 'done'

    def download(self, dst: str):
        """
        Download file to destination.
        """
        self.client().fget_object(
            bucket_name=self.bucket,
            object_name=self.object_name,
            file_path=dst,
        )

    def delete(self, objects: list[str]):
        """
        Delete a list objects from the server.
        """
        try:
            self.client().remove_objects(
                bucket_name=self.bucket,
                delete_object_list=[DeleteObject(obj) for obj in objects],
            )
            logger.info(f'Deleted {len(objects)} from {self.bucket}/{self.prefix}')
        except S3Error as e:
            logger.exception(f'Deleted {len(objects)} from {self.bucket}/{self.prefix}: {e}')

    def scrub(self, max_file_count: int, max_age: pendulum):
        """
        Scrub all files older than maximum age if total file count
        exceeds max_file_count and age exceeds max_age.
        """
        obsolete_files = []

        objects = list(
            self.client().list_objects(
                bucket_name=self.bucket,
                prefix=self.prefix,
                recursive=True,
            )
        )

        if len(objects) <= max_file_count:
            return

        for obj in objects:
            if obj.last_modified < max_age:
                obsolete_files.append(obj.object_name.encode('utf-8'))

        if obsolete_files:
            self.delete(obsolete_files)

    def list_objects(self, prefix):
        """
        Get list of all file objects on the remote server.
        """
        return self.client().list_objects(self.bucket, prefix=prefix, recursive=True)


__all__ = [
    'MinioServer',
]
