from pathlib import Path

from invoke import task

from ..backups import BackupDatabase, get_databases, MinioServer
from ..utils import get_database_service_name


@task
def backup(ctx):
    """
    Backup all databases in the local Postgres cluster.
    """
    for database in get_databases():
        print(f'Backing up database: {database!r}')
        BackupDatabase(ctx, filename=f'{database}.db').process()


@task(
    help={
        'name': 'Database name',
        'interval': 'File archive interval - (daily, weekly, monthly)',
    },
)
def backup_files(ctx, name='', interval=''):
    """
    Show all database backup files for specified database name and interval.
    """
    backups = BackupDatabase(ctx, filename='', interval='').backups(name, interval)

    for interval, db_files in backups.items():
        print(f'\n{interval.title()} backup files')
        for bf in db_files:
            print(bf)
        print()


@task(
    name='list',
    help={'interval': 'File archive interval - (daily, weekly, monthly)'},
)
def db_list(ctx, interval):
    """
    Show all databases in the cluster that have backups for specified interval.
    """
    try:
        databases = BackupDatabase(ctx, filename='', interval=interval).db_list(interval)
    except Exception as e:
        raise SystemExit(f'Unable to get database list - Error:\n\n{e}\n')

    if not databases:
        print(f'\nNo {interval.title()} database backups found\n')
        return

    print(f'\nDatabases with {interval.title()} backups')
    for db in databases:
        print(db)
    print()


@task(
    help={
        'name': 'Database name to restore',
        'filename': 'Backup filename to download',
        'interval': 'File archive interval - (daily, weekly, monthly)',
    },
)
def restore(ctx, name, filename, interval):
    """
    Dump database (if existing) and create from backup file downloaded from the backup server.
    """
    dst = f'/tmp/{filename}'
    database_service = get_database_service_name()

    try:
        MinioServer(filename, prefix=f'databases/{interval}').download(dst)
    except Exception as e:
        raise SystemExit(f'Unable to download {name!r} - Error:\n\n{e}\n')

    if not Path(dst).exists():
        print(f'Unable to download {filename} and restore database {name!r} from backup.')
        return

    # restart database to close database connections
    ctx.run(f'service {database_service} restart')

    print(f'Drop existing {name!r} database')
    ctx.run(f'dropdb -U postgres {name}')
    ctx.run(f'createdb -U postgres {name}')

    print(f'Restoring {name!r} database from backup {filename!r}')
    ctx.run(f'pg_restore -U postgres -d {name} {dst}')


__all__ = (
    'backup',
    'backup_files',
    'db_list',
    'restore',
)
